/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   13.03.2015
 * @brief  Class to represent request from client of access point to SIB.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Class to represent request from client of access point to SIB.
 * Access point uses this class as a wrapper for received request
 * from client to forward it to SIB.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "request.h"

#include <QtCore/QDebug>


QAtomicInt cutesib::access_point::Request::instanceCounter = 0;


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::access_point::Request::Request(AccessPointInterface *accessPoint, QSharedPointer<Client> client,
                                        QString dataProtocol, QByteArray data, QObject *parent)
    : QObject(parent)
{
    accessPoint_ = accessPoint;
    client_ = client;
    dataProtocol_ = dataProtocol;
    data_ = data;

    instanceCounter.ref();
    qDebug() << "Requests:" << instanceCounter;
}


cutesib::access_point::Request::~Request()
{
    instanceCounter.deref();
    qDebug() << "Requests:" << instanceCounter;
}


/*****************************************************************************/
/****************************** Public methods *******************************/

cutesib::access_point::AccessPointInterface *cutesib::access_point::Request::accessPoint()
{
    return accessPoint_;
}


QSharedPointer<cutesib::access_point::Client> cutesib::access_point::Request::client()
{
    return client_;
}


QString cutesib::access_point::Request::dataProtocol() const
{
    return dataProtocol_;
}


const QByteArray cutesib::access_point::Request::data() const
{
    return data_;
}
