/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   01.04.2015
 * @brief  Command class is base class to describe some operations that are performed by modules.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Command is used to describe some operation or task for module.
 * Scheduler manages commands in the SIB. Modules dosen't send commands
 * directly beetwen each other. Commands are sent to scheduler and it
 * gets information from the command to resend the command to appropriate
 * reciever (other module).
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "command.h"


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::Command::Command(CommandType type, QString from, QString to, QObject *parent) : QObject(parent)
{
    type_ = type;
    source_ = from;
    destination_ = to;
}


cutesib::Command::Command(cutesib::CommandType type, QString from, QObject *parent ) : QObject(parent)
{
    type_ = type;
    source_ = from;
    destination_ = QString::null;
}


cutesib::Command::~Command()
{

}


/*****************************************************************************/
/****************************** Public methods *******************************/

void cutesib::Command::setParameter(cutesib::CommandParameter parameter, QVariant value)
{
    parameters_.insert(parameter, value);
}


QVariant cutesib::Command::getParameter(cutesib::CommandParameter parameter)
{
    return parameters_.value(parameter);
}


void cutesib::Command::removesParameter(cutesib::CommandParameter parameter)
{
    parameters_.remove(parameter);
}
