/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   15.03.2015
 * @brief  Logger for SIB.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Simple logger to handle qDebuq, qWarning etc. messages and output information.
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "logger.h"

#include <QtCore/QDebug>
#include <QtCore/QTime>


cutesib::Logger::Logger()
{

}

cutesib::Logger::~Logger()
{

}

QTextStream &cutesib::Logger::message()
{
    static QTextStream ts(stdout);
    return ts << " ";
}

QTextStream &cutesib::Logger::messageNames()
{
    return message() << "SIB_MESSAGE_PREFIX";
}

#if QT_VERSION >= 0x050000
void cutesib::Logger::messageHandler(QtMsgType type, const QMessageLogContext &context, const QString &message)
{
    switch (type) {
    case QtDebugMsg:
        fprintf(stdout, "== %s - [%s-Debug] - %s - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, context.function,
                message.toStdString().c_str());
        fflush(stdout);
        break;
    case QtWarningMsg:
        fprintf(stderr, "== %s - [%s-Warning] - %s - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, context.function,
                message.toStdString().c_str());
        fflush(stderr);
        break;
    case QtCriticalMsg:
        fprintf(stderr, "== %s - [%s-Critical] - %s - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, context.function, message.toStdString().c_str());
        fflush(stderr);
        break;
    case QtFatalMsg:
        fprintf(stderr, "== %s - [%s-Fatal] - %s - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, context.function, message.toStdString().c_str());
        fflush(stderr);
        break;
#if QT_VERSION >= 0x050500
    case QtInfoMsg:
        fprintf(stdout, "== [%s-Message] - %s - %s\n",
                CUTESIB_NAME, context.function, message.toStdString().c_str());
        fflush(stderr);
        break;
#endif

    default:
        break;
    }
}

#else

void cutesib::Logger::messageHandler(QtMsgType type, const char *message)
{
    switch (type) {
    case QtDebugMsg:
        fprintf(stdout, "== %s - [%s-Debug] - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, message);
        fflush(stdout);
        break;
    case QtWarningMsg:
        fprintf(stderr, "== %s - [%s-Warning] - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, message);
        fflush(stderr);
        break;
    case QtCriticalMsg:
        fprintf(stderr, "== %s - [%s-Critical] - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, message);
        fflush(stderr);
        break;
    case QtFatalMsg:
        fprintf(stderr, "== %s - [%s-Fatal] - %s\n",
                QTime::currentTime().toString("hh:mm:ss.zzz").toStdString().c_str(),
                CUTESIB_NAME, message);
        fflush(stderr);
        break;
    default:
        break;
    }
}

#endif
