/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   15.03.2015
 * @brief  Logger for SIB.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Simple logger to handle qDebuq, qWarning etc. messages and output information.
 *
 * Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#ifndef CUTESIB_LOGGER_H
#define CUTESIB_LOGGER_H

#include <QtCore/QTextStream>

#if QT_VERSION >= 0x050000
#include <QtCore/QMessageLogContext>
#endif

#if QT_VERSION < 0x050500
#define qInfo() std::cout
#endif


namespace cutesib
{

class Logger
{
public:
    static QTextStream& message();
    static QTextStream& messageNames();

#if QT_VERSION >= 0x050000
    static void messageHandler(QtMsgType type, const QMessageLogContext &context, const QString &message);
#else
    static void messageHandler(QtMsgType type, const char *message);
#endif


protected:
    Logger();
    virtual ~Logger();
};

}

#endif // CUTESIB_LOGGER_H
