/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   01.04.2015
 * @brief  Scheduler manages commands processing.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * Scheduler manages commands in the SIB. Modules dosen't send commands
 * directly beetwen each other. Commands are sent to scheduler and it
 * gets information from the command to resend the command to appropriate
 * reciever (other module).
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "scheduler.h"

#include <QDebug>

#include "command.h"
#include "commandrequest.h"
#include "commandresponse.h"


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::Scheduler::Scheduler(cutesib::Context *context, QObject *parent) : QObject(parent), context_(context)
{
    qDebug() <<  "Scheduler constructor: done.";
}

cutesib::Scheduler::~Scheduler()
{
    qDebug() <<  "Scheduler destructor: done.";
}


/*****************************************************************************/
/****************************** Public methods *******************************/

cutesib::CommandResponse* cutesib::Scheduler::perform(cutesib::CommandRequest *command)
{
    // Take module by Id or by mapped command type.
    SibModule *module = (command->To().isNull()) ? context_->getModule(command->Type()) : context_->getModule(command->To());

    if (module == nullptr) {


#if __GNUC__ < 5 && __GNUC_MINOR__ < 7
        qWarning() << "Module not found or command type does not mapped to module:"
                   <<  static_cast<int>(command->Type());
#else
        qWarning() << "Module not found or command type does not mapped to module:"
                   << static_cast<std::underlying_type<CommandType>::type>(command->Type());
#endif
        return handleUnmapped(command);
    }
    qDebug() << "MODULE PERFORM";
    return module->perform(command);
}


/*****************************************************************************/
/***************************** Private methods *******************************/

cutesib::CommandResponse* cutesib::Scheduler::handleUnmapped(cutesib::CommandRequest *command)
{
    if (command->needConfirmation()) {
        return new CommandResponse(command, SibError::NOT_MAPPED);
    }

    delete command;

    return nullptr;
}
