#include "sparqlhandler.h"
#include <QDebug>

cutesib::SparqlHandler::SparqlHandler(QObject *parent) : OperationHandler(QString(SPARQLHANDLER_ID), parent)
{

}

cutesib::SparqlHandler::~SparqlHandler()
{
    basicOperationHandler_ = NULL;
}

cutesib::SibError cutesib::SparqlHandler::initialize(Context *context)
{
    SibModule::initialize(context);
    triplestore_ = this->context()->triplestore();
    RDF_world = triplestore_->getRDFWorld();
    RDF_model = triplestore_->getRDFModel();

    basicOperationHandler_ = (BasicOperationHandler*) context->getModule(QString(BASICOPERATIONHANDLER_ID));

    return SibError::NONE;
}

cutesib::CommandResponse *cutesib::SparqlHandler::perform(cutesib::CommandRequest *command)
{
    QString sparqlString = command->getParameter(CommandParameter::QUERY_TEXT).toString();
    //QString sparqlStringTimed =
/*
    gchar * result;

    struct timeval tv;
    gettimeofday(&tv, 0);
    double realtime= (int)tv.tv_sec + ((double)(int)tv.tv_usec / 1000000);
    gchar *realtime_str = g_strdup_printf("%f",realtime);

    result=string_substitution_(sparql_query, get_sib_time, realtime_str);

    g_free(realtime_str);

    return result;
*/

    // Check if sparqlString is SPARQL Query or Update
    bool isSparqlUpdate = false;
    QString firstPart = sparqlString.left(sparqlString.indexOf('{'));
    if (firstPart.contains("insert", Qt::CaseInsensitive) || firstPart.contains("delete", Qt::CaseInsensitive)) {
       isSparqlUpdate = true;
    }

    CommandResponse* response = NULL;

    // SPARQL Query
    if (!isSparqlUpdate) {
        //query=librdf_new_query(world, "sparql", NULL, sparql_query_str, NULL);
        librdf_query* query = librdf_new_query(RDF_world, "sparql", NULL, (unsigned char*) sparqlString.toUtf8().data(), NULL);
        librdf_query_results* results = librdf_model_query_execute(RDF_model, query);
        QString result;

        if (results == NULL)
        {
            //qDebug("SPARQL syntax error! \n");
            librdf_free_query_results(results);
            librdf_free_query(query);
            result = QString("");
            response = new CommandResponse(command, false);
        }
        else if (librdf_query_results_is_graph(results))
        {
            //RESULT GRAPH CASE
            librdf_serializer* serializer;
            size_t length;

            serializer = librdf_new_serializer(RDF_world, NULL, NULL, NULL);

            librdf_stream* stream2 = librdf_query_results_as_stream(results);

            unsigned char* tmpString = librdf_serializer_serialize_stream_to_counted_string(serializer, NULL, stream2, &length);
            result = QString((char*) tmpString).remove(0, 39).simplified(); //WARNING
            librdf_free_memory(tmpString);

            librdf_free_stream(stream2);
            librdf_free_query_results(results);
            librdf_free_query(query);

            response = new CommandResponse(command, true);
            // //librdf_free_serializer(serializer);
            //Redland bug
        }
        else
        {
            //RESULT XML CASE
            //unsigned char* tmpString = librdf_query_results_to_string(results, NULL, NULL);
            unsigned char* tmpString = librdf_query_results_to_string2(results, NULL, NULL, NULL, NULL);
            result = QString((char*) tmpString).remove(0, 39).simplified();
            librdf_free_memory(tmpString);

            librdf_free_query_results(results);
            librdf_free_query(query);

            response = new CommandResponse(command, true);
        }

        response->setParameter(CommandParameter::QUERY_RESULT_TEXT, result);

        return response;
    }
    // SPARQL Update
    else {
        QStringList updateList = sparqlString.split(SPARQL_SEQ_SEPARATOR);
        QStringListIterator listIterator(updateList);
        while(listIterator.hasNext()) {
            QString updateReq = listIterator.next();

            bool hasInsert = false;
            bool hasDelete = false;

            if (updateReq.contains("delete", Qt::CaseInsensitive)) {
                hasDelete = true;
            }

            if (updateReq.contains("insert", Qt::CaseInsensitive)) {
                hasInsert = true;
            }

            if (!hasInsert && !hasDelete) {
                return NULL;
            }

            QString sparqlBeforeWhere;
            QString sparqlAfterWhere;
            QString sparqlStart;

            QString removeStr;
            QString insertStr;

            int startInsert;
            int startRemove;

            QString deleteConstruct;
            QString insertConstruct;

            QList<cutesib::Triple *> deleteTripleList;
            QList<cutesib::Triple *> insertTripleList;

            QStringList resultList = updateReq.split("where", QString::SkipEmptyParts, Qt::CaseInsensitive);
            if (resultList.size() > 1) {
                sparqlBeforeWhere = resultList.at(0);
                sparqlAfterWhere = resultList.at(1);
            }
            else {
                sparqlBeforeWhere = resultList.at(0);
                sparqlBeforeWhere = sparqlBeforeWhere.replace(" data ", "", Qt::CaseInsensitive);
                sparqlAfterWhere = QString(" {}");
            }

            if (hasDelete) {
                startRemove = sparqlBeforeWhere.indexOf("delete", 0, Qt::CaseInsensitive);
                int startRemoveBracket = sparqlBeforeWhere.indexOf('{', startRemove);
                int endRemoveBracket = sparqlBeforeWhere.indexOf('}', startRemoveBracket);
                removeStr = sparqlBeforeWhere.mid(startRemoveBracket, endRemoveBracket - startRemoveBracket);
            }

            if (hasInsert) {
                startInsert = sparqlBeforeWhere.indexOf("insert", 0, Qt::CaseInsensitive);
                int startInsertBracket = sparqlBeforeWhere.indexOf('{', startInsert);
                int endInsertBracket = sparqlBeforeWhere.indexOf('}', startInsertBracket);
                insertStr = sparqlBeforeWhere.mid(startInsertBracket, endInsertBracket - startInsertBracket);
            }

            if (hasDelete && hasInsert) {
                if (startRemove < startInsert)
                    sparqlStart = sparqlBeforeWhere.left(startRemove);
                else
                    sparqlStart = sparqlBeforeWhere.left(startInsert);
            }
            else if (hasDelete)
                sparqlStart = sparqlBeforeWhere.left(startRemove);
            else if (hasInsert)
                sparqlStart = sparqlBeforeWhere.left(startInsert);


            if (hasDelete) {
                deleteConstruct = sparqlStart + QString("CONSTRUCT ") + removeStr +
                        "} WHERE" + sparqlAfterWhere;

                librdf_query *query = librdf_new_query(RDF_world, "sparql", NULL, (unsigned char*) deleteConstruct.toUtf8().data(), NULL);
                librdf_query_results *results = librdf_model_query_execute(RDF_model, query);

                if (results == NULL)
                {
                    librdf_free_query_results(results);
                    librdf_free_query(query);
                }
                else if (librdf_query_results_is_graph(results))
                {
                    librdf_stream *stream = librdf_query_results_as_stream(results);
                    Triple* resultTriple;
                    while(!librdf_stream_end(stream))
                    {
                        librdf_statement *statement = librdf_stream_get_object(stream);
                        ////////////////

                        librdf_node* sub_node  = librdf_statement_get_subject(statement);
                        librdf_node* pred_node = librdf_statement_get_predicate(statement);
                        librdf_node* obj_node  = librdf_statement_get_object(statement);

                        char* sub = (char*) librdf_uri_as_string( librdf_node_get_uri(sub_node));
                        char* pred = (char*) librdf_uri_as_string( librdf_node_get_uri(pred_node));
                        char* obj;

                        if (librdf_node_is_literal(obj_node))
                        {
                            obj = (char*) librdf_node_get_literal_value(obj_node);
                            //triple->objType = ssElement_TYPE_LIT;
                            resultTriple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::LITERAL);
                        }
                        else
                        {
                            obj = (char*) librdf_uri_as_string(librdf_node_get_uri(obj_node));
                            //triple->objType = ssElement_TYPE_URI;
                            resultTriple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::URI);
                        }

                        deleteTripleList.append(resultTriple);

                        //tripleList.append(resultTriple);
                        ////////////////
                        librdf_stream_next(stream);
                    }
                    librdf_free_stream(stream);

                }
                librdf_free_query_results(results);
                librdf_free_query(query);

            }

            if (hasInsert) {
                insertConstruct = sparqlStart + QString("CONSTRUCT ") + insertStr +
                        "} WHERE" + sparqlAfterWhere;

                librdf_query *query = librdf_new_query(RDF_world, "sparql", NULL, (unsigned char*) insertConstruct.toUtf8().data(), NULL);
                librdf_query_results *results = librdf_model_query_execute(RDF_model, query);

                if (results == NULL)
                {
                    librdf_free_query_results(results);
                    librdf_free_query(query);
                }
                else if (librdf_query_results_is_graph(results))
                {
                    librdf_stream *stream = librdf_query_results_as_stream(results);
                    Triple* resultTriple;
                    while(!librdf_stream_end(stream))
                    {
                        librdf_statement *statement=librdf_stream_get_object(stream);
                        ////////////////

                        librdf_node* sub_node  = librdf_statement_get_subject(statement);
                        librdf_node* pred_node = librdf_statement_get_predicate(statement);
                        librdf_node* obj_node  = librdf_statement_get_object(statement);

                        char* sub = (char*) librdf_uri_as_string( librdf_node_get_uri(sub_node));
                        char* pred = (char*) librdf_uri_as_string( librdf_node_get_uri(pred_node));
                        char* obj;

                        if (librdf_node_is_literal(obj_node))
                        {
                            obj = (char*) librdf_node_get_literal_value(obj_node);
                            //triple->objType = ssElement_TYPE_LIT;
                            resultTriple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::LITERAL);
                        }
                        else
                        {
                            obj = (char*) librdf_uri_as_string(librdf_node_get_uri(obj_node));
                            //triple->objType = ssElement_TYPE_URI;
                            resultTriple = new Triple(QString(sub), QString(pred), QString(obj), RdfTripleType::URI);
                        }

                        insertTripleList.append(resultTriple);

                        ////////////////
                        librdf_stream_next(stream);
                    }
                    librdf_free_stream(stream);

                }
                librdf_free_query_results(results);
                librdf_free_query(query);
            }

            if ((deleteTripleList.count() > 0) || (insertTripleList.count() > 0)) {

                CommandRequest* updateRequest = new CommandRequest(CommandType::UPDATE, this->id());

                if (deleteTripleList.count() > 0)
                    updateRequest->setRemoveTriples(deleteTripleList);
                if (insertTripleList.count() > 0)
                    updateRequest->setInsertTriples(insertTripleList);

                if (basicOperationHandler_ != NULL)
                    basicOperationHandler_->update(updateRequest);
            }
        } // end sparql query cycle

        response = new CommandResponse(command, true);
        return response;

    } // end SPARQL Update
}

void cutesib::SparqlHandler::updateState(cutesib::CommandRequest *command)
{

}
