/**
 * @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
 * @date   13.04.2015
 * @brief  Constants for parsing/creating SSAP messages.
 *
 * This file is part of the CuteSIB project.
 *
 * @section LICENSE
 *
 * CuteSIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CuteSIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CuteSIB; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 * @section DESCRIPTION
 *
 * This class is used as a container of constants to parse/create SSAP/XML messages.
 *
 * Contact: http://oss.fruct.org/wiki/SmartSpace-platform
 * Copyright (C) 2015-2016 by PetrSU.
 */


#include "ssapconstants.h"


/*****************************************************************************/
/*************************** Definitions/Constants ***************************/

// SSAP request/response
const QString cutesib::SsapConstants::MessageTypeRequest = "REQUEST";
const QString cutesib::SsapConstants::MessageTypeResponse = "RESPONSE";

// SSAP transactions/messages.
const QString cutesib::SsapConstants::TransactionTypeJoin = "JOIN";
const QString cutesib::SsapConstants::TransactionTypeLeave = "LEAVE";
const QString cutesib::SsapConstants::TransactionTypeInsert = "INSERT";
const QString cutesib::SsapConstants::TransactionTypeUpdate = "UPDATE";
const QString cutesib::SsapConstants::TransactionTypeRemove = "REMOVE";
const QString cutesib::SsapConstants::TransactionTypeQuery = "QUERY";
const QString cutesib::SsapConstants::TransactionTypeSubscribe = "SUBSCRIBE";
const QString cutesib::SsapConstants::TransactionTypeUnsubscribe = "UNSUBSCRIBE";

// SSAP tags.
const QString cutesib::SsapConstants::TagParameter = "parameter";

// SSAP tags for triples.
const QString cutesib::SsapConstants::TagTripleList = "triple_list";
const QString cutesib::SsapConstants::TagTriple = "triple";
const QString cutesib::SsapConstants::TagSubject = "subject";
const QString cutesib::SsapConstants::TagPredicate = "predicate";
const QString cutesib::SsapConstants::TagObject = "object";

// SSAP attributes.
const QString cutesib::SsapConstants::AttributeName = "name";
const QString cutesib::SsapConstants::AttributeEncoding = "encoding";
const QString cutesib::SsapConstants::AttributeType = "type";

// SSAP attributes values.
const QString cutesib::SsapConstants::AttributeValueRdfM3 = "RDF-M3";
const QString cutesib::SsapConstants::ParameterValueSparql = "sparql";
const QString cutesib::SsapConstants::AttributeValueType = "type";
const QString cutesib::SsapConstants::AttributeValueInsertGraph = "insert_graph";
const QString cutesib::SsapConstants::AttributeValueRemoveGraph = "remove_graph";
const QString cutesib::SsapConstants::AttributeValueQueryGraph = "query";
const QString cutesib::SsapConstants::AttributeValueConfirm = "confirm";
const QString cutesib::SsapConstants::AttributeValueSubscriptionId = "subscription_id";

// SSAP attributes values for RDF types.
const QString cutesib::SsapConstants::AttributeValueRdfTypeUri = "uri";
const QString cutesib::SsapConstants::AttributeValueRdfTypeLiteral = "literal";


/*****************************************************************************/
/************************** Constructors/destructors *************************/

cutesib::SsapConstants::SsapConstants()
{

}

cutesib::SsapConstants::~SsapConstants()
{

}
