#include "ssapmessage.h"

#include <QXmlStreamReader>
#include <QStringBuilder>
#include <QDebug>

#include "ssapconstants.h"

#define CUTESIB_SSAP_TAG_TRANSACTION_TYPE "transaction_type"
#define CUTESIB_SSAP_TAG_MESSAGE_TYPE "message_type"
#define CUTESIB_SSAP_TAG_TRANSACTION_ID "transaction_id"
#define CUTESIB_SSAP_TAG_NODE_ID "node_id"
#define CUTESIB_SSAP_TAG_SPACE_ID "space_id"
#define CUTESIB_SSAP_TAG_PARAMETER "parameter"

#define CUTESIB_SSAP_TAG_TRIPLE_LIST "triple_list"
#define CUTESIB_SSAP_TAG_TRIPLE "triple"

#define CUTESIB_SSAP_TAG_PARAMETER "parameter"


#define CUTESIB_SSAP_TAG_ATTRIBUTE_CONFIRM "confirm"


cutesib::SsapMessage::SsapMessage(QObject *parent) : QObject(parent)
{
    xmlDoc = nullptr;
    needConfirmation_ = true;
}

cutesib::SsapMessage::SsapMessage(SsapTransactionType type, cutesib::SsapMessageType messageType, QString nodeId, QString spaceId, QString subscriptionId, QString transactionId, QObject *parent) :
    QObject(parent)
{
    transactionType_ = type;
    needConfirmation_ = true;

    // TODO: make switch and method.
    if (messageType == SsapMessageType::REQUEST) {
        messageType_ = "REQUEST";
    } else if (messageType == SsapMessageType::CONFIRME) {
        messageType_ = "CONFIRM";
    } else if (messageType == SsapMessageType::INDICATION) {
        messageType_ = "INDICATION";
    } else {
        messageType_ = "";
    }

    nodeId_ = nodeId;
    spaceId_ = spaceId;
    subscriptionId_ = subscriptionId;
    transactionId_ = transactionId;
    xmlDoc = nullptr;
}

//FIXME: delete triples
cutesib::SsapMessage::~SsapMessage()
{
    delete xmlDoc;

//    foreach (Triple *t, insertTriples_) {
//        delete t;
//    }

//    foreach (Triple *t, removeTriples_) {
//        delete t;
//    }

//    foreach (Triple *t, resultInsertTriples_) {
//        delete t;
//    }

//    foreach (Triple *t, resultRemoveTriples_) {
//        delete t;
//    }

//    foreach (Triple *t, resultQueryTriples_) {
//        delete t;
//    }
}

bool cutesib::SsapMessage::load(QByteArray message)
{
    delete xmlDoc;

    xmlDoc = new QXmlStreamReader(message);

    while (!xmlDoc->atEnd() && !xmlDoc->hasError()) {

        xmlDoc->readNext();

        if (xmlDoc->isEndElement()) {
            continue;
        }

        if (xmlDoc->name() == CUTESIB_SSAP_TAG_TRANSACTION_TYPE) {
            transactionType_ = convertTrsansactionType(xmlDoc->readElementText());
        } else if (xmlDoc->name() == CUTESIB_SSAP_TAG_MESSAGE_TYPE) {
            messageType_ = xmlDoc->readElementText();
        } else if (xmlDoc->name() == CUTESIB_SSAP_TAG_TRANSACTION_ID) {
            transactionId_ = xmlDoc->readElementText();
        } else if (xmlDoc->name() == CUTESIB_SSAP_TAG_NODE_ID) {
            nodeId_ = xmlDoc->readElementText();
        } else if (xmlDoc->name() == CUTESIB_SSAP_TAG_SPACE_ID) {
            spaceId_ = xmlDoc->readElementText();
        } else if (xmlDoc->name() == SsapConstants::TagParameter) {
            handleParameter(*xmlDoc);
        }
    }

    return true;
}





cutesib::SsapTransactionType cutesib::SsapMessage::transactionType() const
{
    return transactionType_;
}

QString cutesib::SsapMessage::transactionTypeText() const
{
    return transactionTypeText_;
}


QString cutesib::SsapMessage::messageType()
{
    return messageType_;
}


QString cutesib::SsapMessage::nodeId() const
{
    return nodeId_;
}


QString cutesib::SsapMessage::spaceId() const
{
    return spaceId_;
}


QString cutesib::SsapMessage::transactionId() const
{
    return transactionId_;
}


QString cutesib::SsapMessage::subscriptionId() const
{
    return subscriptionId_;
}


void cutesib::SsapMessage::setSubscriptionId(QString id)
{
    subscriptionId_ = id;
}

QByteArray cutesib::SsapMessage::createConfirmation() const
{
    if (transactionType() == SsapTransactionType::JOIN
            || transactionType() == SsapTransactionType::LEAVE
            || transactionType() == SsapTransactionType::REMOVE){

        QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
                           "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
                           "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter></SSAP_message>";

        return QByteArray(confirmation.toUtf8());

    } else if (transactionType() == SsapTransactionType::INSERT
          || transactionType() == SsapTransactionType::UPDATE){

      QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
                         "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
                         "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter>"
                         "<parameter name=\"bnodes\"><urilist></urilist></parameter></SSAP_message>";

      return QByteArray(confirmation.toUtf8());

  } else if (transactionType() == SsapTransactionType::QUERY && !isSparqlQuery()){

        QString triples;
        foreach (Triple *triple, resultQueryTriples_) {
            triples += "<triple><subject type=\"uri\">" + triple->subject()
                    + "</subject><predicate>" + triple->predicate() + "</predicate><object type=";

            if (triple->objectType() == RdfTripleType::LITERAL) {
                triples += "\"literal\"><![CDATA[" + triple->object() + "]]>";
            } else {
                triples += "\"uri\">" + triple->object();
            }

            triples += "</object></triple>";
        }

        QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
                "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
                "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter><parameter name=\"results\">"
                "<triple_list>" + triples + "</triple_list></parameter></SSAP_message>";

        return QByteArray(confirmation.toUtf8());

    } else if (transactionType() == SsapTransactionType::QUERY && isSparqlQuery()){

        QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
                "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
                "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter><parameter name=\"results\">"
                + queryResults_ + "</parameter></SSAP_message>";

        return QByteArray(confirmation.toUtf8());

    } else if (transactionType() == SsapTransactionType::SUBSCRIBE && !isSparqlQuery()){

       QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
               "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
               "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter>"
               "<parameter name=\"subscription_id\">" + subscriptionId_ + "</parameter>"
               "<parameter name=\"results\"><triple_list>" + triplesToSsapList(resultQueryTriples_) + "</triple_list></parameter></SSAP_message>";

       return QByteArray(confirmation.toUtf8());

    } else if (transactionType() == SsapTransactionType::SUBSCRIBE && isSparqlQuery()){

       QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
               "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
               "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter>"
               "<parameter name=\"subscription_id\">" + subscriptionId_ + "</parameter>"
               "<parameter name=\"results\">" + subscriptionNewResults_ + "</parameter></SSAP_message>";

       return QByteArray(confirmation.toUtf8());

    } else if (transactionType() == SsapTransactionType::UNSUBSCRIBE) {

        QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
                "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
                "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:Success</parameter>"
                "<parameter name=\"subscription_id\">" + subscriptionId_ + "</parameter>"
                "</SSAP_message>";

        return QByteArray(confirmation.toUtf8());
    }

    return createErrorConfirmation();
}

QByteArray cutesib::SsapMessage::createErrorConfirmation() const
{
     QString confirmation = "<SSAP_message><message_type>CONFIRM</message_type><transaction_type>" + transactionTypeText() + "</transaction_type>"
           "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
            "<node_id>" + nodeId() + "</node_id><parameter name=\"status\">m3:SIB.Error</parameter>";

     if (transactionType() == SsapTransactionType::UNSUBSCRIBE) {
        confirmation += "<parameter name=\"subscription_id\">" + subscriptionId_ + "</parameter></SSAP_message>";
     } else {
        confirmation += "</SSAP_message>";
     }

     return QByteArray(confirmation.toUtf8());
}

QByteArray cutesib::SsapMessage::createIndication() const
{
    // SSAP INDICATION Example:
    // <SSAP_message><message_type>INDICATION</message_type><transaction_type>SUBSCRIBE</transaction_type><space_id>X</space_id>
    // <node_id>ssls-035f3962-a8b4-4213-bea2-fe96adbe0a4e</node_id>
    // <transaction_id>2</transaction_id>
    // <parameter name="ind_sequence">1</parameter>
    // <parameter name="subscription_id">http://subscribe_graph#ssls-035f3962-a8b4-4213-bea2-fe96adbe0a4e_2</parameter>
    // <parameter name="new_results"><triple_list><triple><subject type="uri">http://test#a</subject><predicate>b</predicate>
    // <object type="uri">c</object></triple></triple_list></parameter>
    // <parameter name="obsolete_results"><triple_list></triple_list></parameter></SSAP_message>

    //TODO: ind sequence.
    QString confirmation;

    if (!isSparqlQuery()) {
        confirmation = "<SSAP_message><message_type>INDICATION</message_type><transaction_type>SUBSCRIBE</transaction_type>"
           "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
           "<node_id>" + nodeId() + "</node_id>"
           "<parameter name=\"subscription_id\">" + subscriptionId_ + "</parameter>"
           "<parameter name=\"ind_sequence\">1</parameter>"
           "<parameter name=\"new_results\"><triple_list>" + triplesToSsapList(resultInsertTriples_) + "</triple_list></parameter>"
           "<parameter name=\"obsolete_results\"><triple_list>" + triplesToSsapList(resultRemoveTriples_) + "</triple_list></parameter>"
           "</SSAP_message>";
    } else {
        confirmation = "<SSAP_message><message_type>INDICATION</message_type><transaction_type>SUBSCRIBE</transaction_type>"
           "<transaction_id>" + transactionId() + "</transaction_id><space_id>" + spaceId() + "</space_id>"
           "<node_id>" + nodeId() + "</node_id>"
           "<parameter name=\"subscription_id\">" + subscriptionId_ + "</parameter>"
           "<parameter name=\"ind_sequence\">1</parameter>"
           "<parameter name=\"new_results\">" + subscriptionNewResults_ + "</parameter>"
           "<parameter name=\"obsolete_results\">" + subscriptionOldResults_ + "</parameter>"
           "</SSAP_message>";
    }

    return QByteArray(confirmation.toUtf8());
}






cutesib::SsapTransactionType cutesib::SsapMessage::convertTrsansactionType(QString transactionType)
{
    transactionTypeText_ = transactionType;

    if (transactionType == SsapConstants::TransactionTypeJoin) {
        return SsapTransactionType::JOIN;
    }

    if (transactionType == SsapConstants::TransactionTypeLeave) {
        return SsapTransactionType::LEAVE;
    }

    if (transactionType == SsapConstants::TransactionTypeInsert) {
        return SsapTransactionType::INSERT;
    }

    if (transactionType == SsapConstants::TransactionTypeQuery) {
        return SsapTransactionType::QUERY;
    }

    if (transactionType == SsapConstants::TransactionTypeUpdate) {
        return SsapTransactionType::UPDATE;
    }

    if (transactionType == SsapConstants::TransactionTypeRemove) {
        return SsapTransactionType::REMOVE;
    }

    if (transactionType == SsapConstants::TransactionTypeSubscribe) {
        return SsapTransactionType::SUBSCRIBE;
    }

    if (transactionType == SsapConstants::TransactionTypeUnsubscribe) {
        return SsapTransactionType::UNSUBSCRIBE;
    }

    // Clean text if the transaction is unknown.
    transactionTypeText_.clear();

    return SsapTransactionType::UNKNOWN;
}

// Handle <parameter name =...
void cutesib::SsapMessage::handleParameter(QXmlStreamReader &doc)
{
    QXmlStreamAttributes attributes = doc.attributes();

    qDebug() << "Attributes" << attributes.count() << attributes.value(SsapConstants::AttributeName);

    QStringRef name = attributes.value(SsapConstants::AttributeName);

    if (name == SsapConstants::AttributeValueConfirm) {
        readNotNullNext();
        needConfirmation_ = (doc.readElementText().compare("TRUE", Qt::CaseInsensitive)) ? true : false;
    } else if (name == SsapConstants::AttributeValueInsertGraph
               && attributes.value(SsapConstants::AttributeEncoding) == SsapConstants::AttributeValueRdfM3) {

        // Handle SSAP triples list (encoding="RDF-M3").
        doc.readNext();

        if (doc.name() == SsapConstants::TagTripleList) {
            qDebug() << "Insert triples list handling.";
            insertTriples_ = handleTripleList(doc);
        } else {
            doc.readNextStartElement();
        }

    } else if (name == SsapConstants::AttributeValueRemoveGraph
               && attributes.value(SsapConstants::AttributeEncoding) == SsapConstants::AttributeValueRdfM3) {

        // Handle SSAP triples list (encoding="RDF-M3").
        doc.readNext();

        if (doc.name() == SsapConstants::TagTripleList) {
            qDebug() << "Remove triples list handling.";
            removeTriples_ = handleTripleList(doc);
        } else {
            doc.readNextStartElement();
        }

    }  else if (name == SsapConstants::AttributeValueQueryGraph) {
                //&& attributes.value(SsapConstants::AttributeEncoding) == SsapConstants::AttributeValueRdfM3) {

         // Handle SSAP triples list (encoding="RDF-M3") or sparql.
         doc.readNext();

         if (doc.name() == SsapConstants::TagTripleList) {
             qDebug() << "Query triples list handling.";
             queryTriples_ = handleTripleList(doc);
         } if (doc.isCharacters() || doc.isCDATA()) {
             query_ = doc.text().toString();
         }

         else {
             doc.readNextStartElement();
         }

    } else if (name == SsapConstants::AttributeValueSubscriptionId) {

        subscriptionId_ = doc.readElementText();

    } else if (name == SsapConstants::AttributeValueType) {

        type_ = doc.readElementText();

    } else {
        doc.readNextStartElement();
    }


//    foreach (QXmlStreamAttribute attribute, doc.attributes()) {
//        if (attribute.name() == CUTESIB_SSAP_TAG_ATTRIBUTE_CONFIRM) {
//           isNeedConfirmation_ = true;
//        } else if (attribute.name() == CUTESIB_SSAP_TAG_ATTRIBUTE_INSERT_GRAPH) {
//            doc.readNext();

//            if (doc.name() == CUTESIB_SSAP_TAG_TRIPLE_LIST) {
//                qDebug() << doc.name();
//                if (doc.name() == CUTESIB_SSAP_TAG_TRIPLE) {
//                    //qDebug() <<
//                }

//            } else {
//                doc.readNextStartElement();
//            }

//        }
//    }


}

QList<cutesib::Triple*> cutesib::SsapMessage::handleTripleList(QXmlStreamReader &doc)
{
    QList<Triple*> triples;


    while (!doc.atEnd() && !doc.hasError()) {

        // For QT parser spaces (<triple>   <subject>) are sepated elements
        // with NULL name, skeep spaces.
        do {
            doc.readNext();
        } while (doc.name() == nullptr);

        if (doc.name() == SsapConstants::TagTriple
                && doc.isStartElement()) {
            Triple *triple = handleTriple(doc);

            if (triple) {
                qDebug() << "Created triple:" << triple->subject() << "-" << triple->predicate() << "-" << triple->object();
                triples.append(triple);
            }
        } else if (doc.name() == SsapConstants::TagTriple
                && doc.isEndElement()) {
            doc.readNext();
        } else {
            break;
        }
    }

    return triples;
}

cutesib::Triple* cutesib::SsapMessage::handleTriple(QXmlStreamReader &doc)
{
    QString subject = QString::null;
    QString object = QString::null;
    QString predicate = QString::null;
    RdfTripleType objectType = RdfTripleType::URI;

    // For QT parser spaces (<triple>   <subject>) are sepated elements
    // with NULL name, skeep spaces.
    do {
        doc.readNext();
    } while (doc.name() == nullptr);

    while (true) {
        qDebug() << doc.name() << doc.tokenString() << doc.attributes().count() << getAttribute(doc, "type") << getRdfType(doc);

        if (doc.name() == "subject") {
            subject = doc.readElementText();
        } else if (doc.name() == "object") {
            objectType = getRdfType(doc);
            object = doc.readElementText();
        } else if (doc.name() == "predicate") {
            predicate = doc.readElementText();
        } else {
            break;
        }

        // For QT parser spaces (<triple>   <subject>) are sepated elements
        // with NULL name, skeep spaces.
        do {
            doc.readNext();
        } while (doc.name() == nullptr);
    }

    if (subject.isNull()
            || predicate.isNull()
            || object.isNull()
            || objectType == RdfTripleType::UNKNOWN) {
        return 0;
    }

    return new Triple(subject, predicate, object, objectType);
}

QString cutesib::SsapMessage::getAttribute(QXmlStreamReader &doc, QString name)
{
    foreach (QXmlStreamAttribute attribute, doc.attributes()) {
        if (attribute.name() == name){
            return attribute.value().toString();
        }
    }

    return QString::null;
}

cutesib::RdfTripleType cutesib::SsapMessage::getRdfType(QXmlStreamReader &doc)
{
    QStringRef type = doc.attributes().value(SsapConstants::AttributeType); //getAttribute(doc, "type");

    if (QStringRef::compare(type, SsapConstants::AttributeValueRdfTypeUri, Qt::CaseInsensitive) == 0) {
        return RdfTripleType::URI;
    } else if (QStringRef::compare(type, SsapConstants::AttributeValueRdfTypeLiteral, Qt::CaseInsensitive) == 0) {
        return RdfTripleType::LITERAL;
    }

    return RdfTripleType::UNKNOWN;
}

QString cutesib::SsapMessage::triplesToSsapList(QList<Triple *> triples)
{
    QString triplesText;

    foreach (Triple *triple, triples) {
        triplesText = triplesText % "<triple><subject type=\"uri\">" % triple->subject()
                        % "</subject><predicate>" % triple->predicate() % "</predicate><object type=";

        if (triple->objectType() == RdfTripleType::LITERAL) {
            triplesText = triplesText % "\"literal\"><![CDATA[" % triple->object() % "]]>";
        } else {
            triplesText = triplesText % "\"uri\">" % triple->object();
        }

        triplesText = triplesText % "</object></triple>";
    }

    return triplesText;
}

void cutesib::SsapMessage::readNotNullNext()
{
    // For QT parser spaces (<triple>   <subject>) are sepated elements
    // with NULL name, skeep spaces.
    do {
        xmlDoc->readNext();
    } while (xmlDoc->name() == nullptr);
}

QString cutesib::SsapMessage::parameterType() const
{
    return type_;
}

QString cutesib::SsapMessage::parameterQuery() const
{
    return query_;
}

bool cutesib::SsapMessage::isSparqlQuery() const
{
    return SsapConstants::ParameterValueSparql.compare(type_, Qt::CaseInsensitive) == 0;
}


bool cutesib::SsapMessage::isNeedConfirmation() const
{
    return needConfirmation_;
}


const QList<cutesib::Triple *>  cutesib::SsapMessage::insertTriples() const
{
    return insertTriples_;
}

const QList<cutesib::Triple *> cutesib::SsapMessage::removeTriples() const
{
    return removeTriples_;
}

const QList<cutesib::Triple *> cutesib::SsapMessage::queryTriples() const
{
    return queryTriples_;
}

void cutesib::SsapMessage::setResultInsertTriples(QList<cutesib::Triple *> triples)
{
    foreach (Triple *t, resultInsertTriples_) {
        delete t;
    }

    resultInsertTriples_ = triples;
}


void cutesib::SsapMessage::setResultRemoveTriples(QList<cutesib::Triple *> triples)
{
    foreach (Triple *t, resultRemoveTriples_) {
        delete t;
    }

    resultRemoveTriples_ = triples;
}

void cutesib::SsapMessage::setResultQueryTriples(QList<cutesib::Triple *> triples)
{
    qDebug() << "before delete";
    foreach (Triple *t, resultQueryTriples_) {
        delete t;
    }
qDebug() << "after delete";
    foreach (Triple *t, triples) {
        resultQueryTriples_.append(t);
    }


}
