/**
* @author Ivan V. Galov <galov@cs.karelia.ru>
* @date   18.04.2016
* @brief  Class containing information about one subscription.
*
* This file is part of the CuteSIB project.
*
* @section LICENSE
*
* CuteSIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* CuteSIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with CuteSIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Class containing information about one subscription.
*
* Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
* Copyright (C) 2015-2016 by PetrSU.
**/

#ifndef SUBSCRIPTION_H
#define SUBSCRIPTION_H

#include <redland.h>
#include <rasqal.h>

#include <QList>
#include "triple.h"

namespace cutesib {

typedef struct {
QList<Triple*> insertedTriples;
QList<Triple*> removedTriples;
} SubscriptionIndication;

class Subscription
{
public:
    Subscription(QString subscriptionId, QList<Triple *> queryList);
    ~Subscription();
    QString getSubscriptionId();

    void checkSubscription(QList<Triple*> insertedTriplesList, QList<Triple*> removedTriplesList, SubscriptionIndication& indication);

    void addTripleListInSubmodel(QList<librdf_statement*> tripleList);
    void addTripleListInSubmodel(QList<Triple*> tripleList);

private:
    librdf_world* subworld_;
    librdf_storage* substorage_;
    librdf_model* submodel_;

    QList<Triple*> queryList_;

    QString subscriptionId_;
};

}

#endif // SUBSCRIPTION_H
