/**
* @author Ivan V. Galov <galov@cs.karelia.ru>
* @date   18.04.2016
* @brief  Class for creating and working with Redland triplestore.
*
* This file is part of the CuteSIB project.
*
* @section LICENSE
*
* CuteSIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* CuteSIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with CuteSIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Class for creating and working with Redland triplestore.
*
* Web-site: http://oss.fruct.org/wiki/SmartSpace-platform
* Copyright (C) 2015-2016 by PetrSU.
**/

#ifndef TRIPLESTORE_H
#define TRIPLESTORE_H

#include <redland.h>
#include <rasqal.h>
#include "sibconfig.h"

class Triplestore
{

public:
    Triplestore(cutesib::SibConfig *config);
    ~Triplestore();
    void initialize();
    void resetStorageModel();
    void removeAllStatements();
    librdf_world* getRDFWorld();
    librdf_model* getRDFModel();
    cutesib::SibConfig* getConfig();

private:
    QString ssType_;
    QString ssName_;
    QString ssPath_;

    QString dbHost_;
    QString dbUser_;
    QString dbPassword_;

    librdf_world* RDF_world;
    librdf_storage* RDF_storage;
    librdf_model* RDF_model;
    cutesib::SibConfig* config_;
};

#endif // TRIPLESTORE_H
